#include <stdio.h>
#include <string.h>
#include "gendlist.h"


typedef struct item_s item_t;


struct item_s {
	int id;
	char *payload;
	gdl_elem_t l1;
	gdl_elem_t l2;
};

/* initialized to all NULL */
gdl_list_t list1, list2;

static item_t *new_item(const char *payload)
{
	static int ids = 1;
	item_t *i;
	i = calloc(sizeof(item_t), 1);
	i->id = ids++;
	i->payload = strdup(payload);
	return i;
}

const char * ptr2id(item_t *i)
{
	static char buff[32][32];
	static int nextbuff = 0;
	char *b;

	b = buff[(nextbuff++)%32];
	
	if (i == NULL)
		sprintf(b, "#!");
	else
		sprintf(b, "#%d", i->id);
	return b;
}

static void dump_list1(const char *banner)
{
	gdl_iterator_t it;
	item_t *i;

	printf("\n%s {length=%d}\n", banner, list1.length);

	gdl_foreach(&list1, &it, i) {
		printf(" [%d] %s (%s, %s): %s \n", gdl_it_idx(&it), ptr2id(i), ptr2id(i->l1.prev), ptr2id(i->l1.next), i->payload);
	}
	gdl_check(&list1);
}

int main()
{
	int n;
	item_t *i, *a;

	for(n = 0; n < 4; n++) {
		char tmp[128];
		sprintf(tmp, "item %d", n);
		i = new_item(tmp);
		gdl_append(&list1, i, l1);
	}

	i = new_item("woops");
	gdl_insert(&list1, i, l1);

	for(n = 0; n < 5; n++) {
		gdl_nth(&list1, n, &i);
		printf("[%d] %s\n", n, i->payload);
	}

	dump_list1("initial");


	gdl_nth(&list1, 2, &a);
	i = new_item("before-1");
	gdl_insert_before(&list1, a, i, l1);
	i = new_item("after-1");
	gdl_insert_after(&list1, a, i, l1);
	dump_list1("insert-at");

	gdl_remove(&list1, a, l1);
	dump_list1("remove 1");
	gdl_remove(&list1, i, l1);
	dump_list1("remove after");

	a = list1.first;
	gdl_remove(&list1, a, l1);
	dump_list1("remove first");

	a = list1.last;
	gdl_remove(&list1, a, l1);
	dump_list1("remove last");

	for(;;) {
		a = list1.last;
		if (a == NULL)
			break;
		gdl_remove(&list1, a, l1);
		dump_list1("remove cyc");
	}

}
