#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "itemlist.h"

itemlist_t lst;

item_t *alloc_item(const char *payload)
{
	static int id = 1;
	item_t *a;

	a = malloc(sizeof(item_t));
	a->id = id++;
	strcpy(a->payload, payload);
	return a;
}

void print_item(const char *prefix, item_t *i, const char *suffix)
{
	printf("%s[%d]='%s'%s", prefix, i->id, i->payload, suffix);
}

void print_list(itemlist_t *lst, const char *prefix, int res)
{
	item_t *i;
	gdl_iterator_t it;

	printf("%s", prefix);
	itemlist_foreach(lst, &it, i) {
		print_item(" ", i, "");
	}
	printf(" (%d)\n", res);
}

int cmp_mod(item_t *i, void *data)
{
	int *mod = data;
	if ((i->id % 2) == *mod)
		return 0;
	return 1;
}

int main()
{
	int n, res, zero = 0;
	item_t *i;

	/* create a list of 4 items */
	for(n = 0; n < 4; n++) {
		char buff[16];
		sprintf(buff, "It%d", 101+n);
		itemlist_append(&lst, alloc_item(buff));
	}
	print_list(&lst, "init:", 0);

	/* insert a new item in front of the list */
	i = alloc_item("foo");
	res = itemlist_insert(&lst, i);
	print_list(&lst, "ins1:", res);

	/* these would fail, as i is already inserted */
	res = itemlist_insert(&lst, i);
	print_list(&lst, "ins2:", res);
	res = itemlist_append(&lst, i);
	print_list(&lst, "app: ", res);

	/* retrieve an item */
	i = itemlist_nth(&lst, 3);
	print_item("3rd:  ", i, "\n");

	/* search every element with even ID */
	for(i = itemlist_find(&lst, &zero, cmp_mod); i != NULL; i = itemlist_find_next(i, &zero, cmp_mod)) {
		print_item("find: ", i, "\n");
	}

	/* reverse */
	itemlist_reverse(&lst);
	print_list(&lst, "rev: ", 0);


	/* Check the itegrity of the list */
	printf("chk:  %d\n", itemlist_check(&lst));

	return 0;
}

